@@ -1,25 +1,20 @@
-/**************************************************************************
- *  Kchmviewer - a portable CHM file viewer with the best support for     *
- *  the international languages                                           *
- *                                                                        *
- *  Copyright (C) 2004-2012 George Yunaev, kchmviewer@ulduzsoft.com       *
- *                                                                        *
- *  Please read http://www.kchmviewer.net/reportbugs.html if you want     *
- *  to report a bug. It lists things I need to fix it!                    *
- *                                                                        *
- *  This program is free software: you can redistribute it and/or modify  *
- *  it under the terms of the GNU General Public License as published by  *
- *  the Free Software Foundation, either version 3 of the License, or     *
- *  (at your option) any later version.                                   *
- *																	      *
- *  This program is distributed in the hope that it will be useful,       *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *  GNU General Public License for more details.                          *
- *                                                                        *
- *  You should have received a copy of the GNU General Public License     *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- **************************************************************************/
+/*
+ *  Kchmviewer - a CHM and EPUB file viewer with broad language support
+ *  Copyright (C) 2004-2014 George Yunaev, gyunaev@ulduzsoft.com
+ *
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "kde-qt.h"
 #include "config.h"
@@ -57,6 +52,7 @@ DialogSetup::DialogSetup(QWidget *parent)
 	m_radioExtLinkOpenAlways->setChecked ( pConfig->m_onExternalLinkClick == Config::ACTION_ALWAYS_OPEN );
 	m_radioExtLinkAsk->setChecked ( pConfig->m_onExternalLinkClick == Config::ACTION_ASK_USER );
 	m_radioExtLinkOpenNever->setChecked ( pConfig->m_onExternalLinkClick == Config::ACTION_DONT_OPEN );
+    m_highlightSearchResults->setChecked( pConfig->m_browserHighlightSearchResults );
 	
 	m_radioNewChmOpenAlways->setChecked ( pConfig->m_onNewChmClick == Config::ACTION_ALWAYS_OPEN );
 	m_radioNewChmAsk->setChecked ( pConfig->m_onNewChmClick == Config::ACTION_ASK_USER );
@@ -78,6 +74,7 @@ DialogSetup::DialogSetup(QWidget *parent)
 	m_enablePlugins->setChecked( pConfig->m_browserEnablePlugins );
 	m_enableOfflineStorage->setChecked( pConfig->m_browserEnableOfflineStorage );
 	m_enableLocalStorage->setChecked( pConfig->m_browserEnableLocalStorage );
+    m_openAllTOCEntries->setChecked( pConfig->m_tocOpenAllEntries );
 
 	switch ( pConfig->m_toolbarMode )
 	{
@@ -126,6 +123,8 @@ void DialogSetup::accept()
 
 	pConfig->m_numOfRecentFiles = m_historySize->value();
 	pConfig->m_HistoryStoreExtra = m_rememberHistoryInfo->isChecked();
+    pConfig->m_browserHighlightSearchResults = m_highlightSearchResults->isChecked();
+    pConfig->m_tocOpenAllEntries = m_openAllTOCEntries->isChecked();
 
 	if ( m_radioExtLinkOpenAlways->isChecked () )
 		pConfig->m_onExternalLinkClick = Config::ACTION_ALWAYS_OPEN;
