@@ -1,59 +1,59 @@
-/**************************************************************************
- *  Kchmviewer - a portable CHM file viewer with the best support for     *
- *  the international languages                                           *
- *                                                                        *
- *  Copyright (C) 2004-2012 George Yunaev, kchmviewer@ulduzsoft.com       *
- *                                                                        *
- *  Please read http://www.kchmviewer.net/reportbugs.html if you want     *
- *  to report a bug. It lists things I need to fix it!                    *
- *                                                                        *
- *  This program is free software: you can redistribute it and/or modify  *
- *  it under the terms of the GNU General Public License as published by  *
- *  the Free Software Foundation, either version 3 of the License, or     *
- *  (at your option) any later version.                                   *
- *																	      *
- *  This program is distributed in the hope that it will be useful,       *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *  GNU General Public License for more details.                          *
- *                                                                        *
- *  You should have received a copy of the GNU General Public License     *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- **************************************************************************/
+/*
+ *  Kchmviewer - a CHM and EPUB file viewer with broad language support
+ *  Copyright (C) 2004-2014 George Yunaev, gyunaev@ulduzsoft.com
+ *
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <QProcess>
 #include <QDesktopServices>
 #include <QSettings>
 #include <QDateTime>
+#include <QTextStream>
 
 #include "kde-qt.h"
 
-#include "libchmfile.h"
-#include "libchmfileimpl.h"
-#include "libchmurlfactory.h"
-
 #include "mainwindow.h"
 #include "config.h"
-#include "treeviewitem.h"
 #include "settings.h"
 #include "viewwindow.h"
 #include "viewwindowmgr.h"
-#include "keyeventfilter.h"
 #include "dialog_setup.h"
 #include "recentfiles.h"
 #include "navigationpanel.h"
 #include "toolbarmanager.h"
 #include "version.h"
+#include "textencodings.h"
 #include "ui_dialog_about.h"
 
+#ifdef Q_WS_X11
+    #include <QX11Info>
+    #include <X11/Xlib.h>
+#endif
 
-MainWindow::MainWindow()
+// Maximum memory size for inter-application communication
+static const int SHARED_MEMORY_SIZE = 4096;
+
+MainWindow::MainWindow( const QStringList& arguments )
 	: QMainWindow ( 0 ), Ui::MainWindow()
 {
 	const unsigned int WND_X_SIZE = 900;
 	const unsigned int WND_Y_SIZE = 700;
 	const unsigned int SPLT_X_SIZE = 300;
 
+    m_arguments = arguments;
+
 	// Delete the pointer when the window is closed
 	setAttribute( Qt::WA_DeleteOnClose );
 	
@@ -66,8 +66,9 @@ MainWindow::MainWindow()
 	else
 		qApp->setLayoutDirection( Qt::LeftToRight );
 	
-	m_chmFile = 0;
+	m_ebookFile = 0;
 	m_autoteststate = STATE_OFF;
+    m_sharedMemory = 0;
 
 	m_currentSettings = new Settings();
 		
@@ -114,22 +115,6 @@ MainWindow::MainWindow()
 
 	// Basically disable everything
 	updateActions();
-
-	// Check for a new version if needed
-	if ( pConfig->m_advCheckNewVersion )
-	{
-		QSettings settings;
-
-		if ( settings.contains( "advanced/lastupdate" ) )
-		{
-			QDateTime lastupdate = settings.value( "advanced/lastupdate" ).toDateTime();
-
-			if ( lastupdate.secsTo( QDateTime::currentDateTime() ) >= 86400 * 7 ) // seven days
-				checkNewVersionAvailable();
-		}
-	}
-
-	QTimer::singleShot( 0, this, SLOT( firstShow()) );
 }
 
 MainWindow::~MainWindow()
@@ -137,6 +122,106 @@ MainWindow::~MainWindow()
 	// Temporary files cleanup
 	while ( !m_tempFileKeeper.isEmpty() )
 		delete m_tempFileKeeper.takeFirst();
+
+    delete m_sharedMemory;
+}
+
+void MainWindow::launch()
+{
+    QTimer::singleShot( 0, this, SLOT( firstShow()) );
+}
+
+bool MainWindow::hasSameTokenInstance()
+{
+    // Find out if token has been specified as this would mean we're running in a single instance mode
+    QString token;
+
+    // argv[0] in Qt is still a program name
+    for ( int i = 1; i < m_arguments.size(); i++  )
+    {
+        // This is not bulletproof (think -showPage -token) but this is not likely to happen
+        if ( m_arguments[i] == "-token")
+        {
+            token = m_arguments[++i];
+            break;
+        }
+    }
+
+    if ( token.isEmpty() )
+        return false;
+
+    m_sharedMemory = new QSharedMemory( token );
+
+    // If we can attach to it, the segment already exists
+    if ( m_sharedMemory->attach() )
+    {
+        // Another instance exists; send the command-line there
+        QByteArray args = m_arguments.join("|").toLocal8Bit();
+
+        if ( args.size() < SHARED_MEMORY_SIZE - 2 )
+        {
+            // Write the size first, then the string
+            if ( m_sharedMemory->lock() )
+            {
+                char * data = (char*) m_sharedMemory->data();
+                *((short*)data) = args.size();
+                memcpy( data + 2, args.data(), args.size() );
+
+                m_sharedMemory->unlock();
+            }
+            else
+                qDebug("failed to lock");
+        }
+
+        // Clean up
+        delete m_sharedMemory;
+        m_sharedMemory = 0;
+
+        return true;
+    }
+
+    // Create a new segment
+    if ( !m_sharedMemory->create( SHARED_MEMORY_SIZE ) )
+    {
+        QMessageBox::critical( 0,
+                               i18n("Shared memory segment failed"),
+                               i18n("Failed to create a shared memory segment: %1").arg( m_sharedMemory->errorString()) );
+        return false;
+    }
+
+    // Set it up so our checker knows there's no data yet
+    *((short*) m_sharedMemory->data()) = 0;
+
+    // Recheck every second
+    QTimer * timer = new QTimer( this );
+    connect( timer, SIGNAL(timeout()), this, SLOT(checkForSharedMemoryMessage()));
+    timer->start(1000);
+    return false;
+}
+
+void MainWindow::checkForSharedMemoryMessage()
+{
+    QStringList args;
+    m_sharedMemory->lock();
+
+    // Is there any data?
+    char * data = (char*) m_sharedMemory->data();
+
+    if ( data[0] != 0 || data[1] != 0 )
+    {
+        // Get the message length and the message
+        short len = *((short*) data);
+        args = QString::fromLocal8Bit( data + 2, len ).split( "|" );
+
+        // Clean up
+        *((short*) data) = 0;
+    }
+
+    m_sharedMemory->unlock();
+
+    // And process it if we find anything
+    if ( !args.isEmpty() )
+        parseCmdLineArgs( args, true );
 }
 
 void MainWindow::checkNewVersionAvailable()
@@ -160,41 +245,42 @@ bool MainWindow::loadFile ( const QString &loadFileName, bool call_open_page )
 	if ( fileName.startsWith( "file://" ) )
 		fileName.remove( 0, 7 );
 			
-	LCHMFile * new_chmfile = new LCHMFile();
+	EBook * new_ebook = EBook::loadFile( fileName );
 	
-	if ( new_chmfile->loadFile( fileName ) )
+	if ( new_ebook )
 	{
 		// The new file is opened, so we can close the old one
-		if ( m_chmFile )
+		if ( m_ebookFile )
 		{
 			closeFile( );
-			delete m_chmFile;
+			delete m_ebookFile;
 		}
 	
-		m_chmFile = new_chmfile;
+		m_ebookFile = new_ebook;
 		updateActions();
 		
 		// Show current encoding in status bar
-		showInStatusBar( i18n("Detected file encoding: %1 ( %2 )") 
-		                 .arg( m_chmFile->currentEncoding()->family)
-		                 .arg( m_chmFile->currentEncoding()->qtcodec) );
+		if ( m_ebookFile->hasFeature( EBook::FEATURE_ENCODING ) )
+			showInStatusBar( i18n("Detected file encoding: %1 ( %2 )")
+							 .arg( TextEncodings::languageForCodec( m_ebookFile->currentEncoding() ))
+							 .arg( m_ebookFile->currentEncoding() ) );
 
 		// Make the file name absolute; we'll need it later
 		QDir qd;
 		qd.setPath (fileName);
-		m_chmFilename = qd.absolutePath();
+		m_ebookFilename = qd.absolutePath();
 		
 		// Qt's 'dirname' does not work well
-		QFileInfo qf ( m_chmFilename );
+		QFileInfo qf ( m_ebookFilename );
 		pConfig->m_lastOpenedDir = qf.dir().path();
-		m_chmFileBasename = qf.fileName();
+		m_ebookFileBasename = qf.fileName();
 
 		// Apply settings to the navigation dock
-		m_navPanel->updateTabs( m_chmFile );
+		m_navPanel->updateTabs( m_ebookFile );
 
 		// and to navigation buttons
-		nav_actionPreviousPage->setEnabled( m_chmFile->hasTableOfContents() );
-		nav_actionNextPageToc->setEnabled( m_chmFile->hasTableOfContents() );
+		nav_actionPreviousPage->setEnabled( hasTableOfContents()  );
+		nav_actionNextPageToc->setEnabled( hasTableOfContents() );
 
 		navSetBackEnabled( false );
 		navSetForwardEnabled( false );
@@ -204,11 +290,8 @@ bool MainWindow::loadFile ( const QString &loadFileName, bool call_open_page )
 
 		if ( m_currentSettings->loadSettings (fileName) )
 		{
-			const LCHMTextEncoding * encoding = 
-					m_chmFile->impl()->lookupByQtCodec(  m_currentSettings->m_activeEncoding );
-
-			if ( encoding )
-				setTextEncoding( encoding );
+			if ( m_ebookFile->hasFeature( EBook::FEATURE_ENCODING ) )
+				setTextEncoding(m_currentSettings->m_activeEncoding );
 
 			m_navPanel->applySettings( m_currentSettings );
 			
@@ -217,7 +300,7 @@ bool MainWindow::loadFile ( const QString &loadFileName, bool call_open_page )
 				m_viewWindowMgr->restoreSettings( m_currentSettings->m_viewwindows );
 				m_viewWindowMgr->setCurrentPage( m_currentSettings->m_activetabwindow );
 				
-				if ( m_chmFile->hasTableOfContents() )
+				if ( m_ebookFile->hasFeature( EBook::FEATURE_TOC ) )
 					actionLocateInContentsTab();
 			}
 			
@@ -229,14 +312,16 @@ bool MainWindow::loadFile ( const QString &loadFileName, bool call_open_page )
 		else
 		{
 			m_navPanel->setActive( NavigationPanel::TAB_CONTENTS );
-			setTextEncoding( m_chmFile->currentEncoding() );
+
+			if ( m_ebookFile->hasFeature( EBook::FEATURE_ENCODING ) )
+				setTextEncoding( m_ebookFile->currentEncoding() );
 			
 			if ( call_open_page )
-				openPage( m_chmFile->homeUrl() );
+				openPage( m_ebookFile->homeUrl() );
 		}
 
 		if ( m_recentFiles )
-			m_recentFiles->setCurrentFile( m_chmFilename );
+			m_recentFiles->setCurrentFile( m_ebookFilename );
 
 		return true;
 	}
@@ -255,16 +340,14 @@ bool MainWindow::loadFile ( const QString &loadFileName, bool call_open_page )
 				i18n("Could not load file %1").arg(fileName),
 				2000 );
 
-		delete new_chmfile;
 		return false;
 	}
 }
 
 
-
 void MainWindow::refreshCurrentBrowser( )
 {
-	QString title = m_chmFile->title();
+	QString title = m_ebookFile->title();
 	
 	if ( title.isEmpty() )
 		title = QCoreApplication::applicationName();
@@ -283,32 +366,27 @@ void MainWindow::refreshCurrentBrowser( )
 
 
 void MainWindow::activateUrl( const QUrl & link )
-{
-	activateLink( link.toString() );
-}
-
-void MainWindow::activateLink ( const QString & link )
 {
 	if ( link.isEmpty() )
 		return;
-	
-	if ( gKeyEventFilter.isShiftPressed() )
+
+	Qt::KeyboardModifiers mods = QApplication::keyboardModifiers();
+
+	if ( mods & Qt::ShiftModifier )
 		openPage( link, OPF_NEW_TAB | OPF_CONTENT_TREE );
-	else if ( gKeyEventFilter.isCtrlPressed() )
+	else if ( mods & Qt::ControlModifier )
 		openPage( link, OPF_NEW_TAB | OPF_BACKGROUND );
 	else
-		openPage( link, OPF_CONTENT_TREE | OPF_ADD2HISTORY );
+		openPage( link, OPF_CONTENT_TREE );
 }
 
 
-bool MainWindow::openPage( const QString & srcurl, unsigned int flags )
+bool MainWindow::openPage( const QUrl& url, unsigned int flags )
 {
-	QString otherlink, otherfile, url = srcurl;
+	QString otherlink;
 	
-	if ( url == "/" )
-		url = m_chmFile->homeUrl();
-
-	if ( LCHMUrlFactory::isRemoteURL (url, otherlink) )
+	// Feed to the browser all non-internal URLs
+	if ( !m_ebookFile->isSupportedUrl( url  ) )
 	{
 		switch ( pConfig->m_onExternalLinkClick )
 		{
@@ -318,7 +396,7 @@ bool MainWindow::openPage( const QString & srcurl, unsigned int flags )
 		case Config::ACTION_ASK_USER:
 	   		if ( QMessageBox::question(this,
 				 i18n("%1 - remote link clicked - %2") . arg(QCoreApplication::applicationName()) . arg(otherlink),
-				 i18n("A remote link %1 will start the external program to open it.\n\nDo you want to continue?").arg( url ),
+				 i18n("A remote link %1 will start the external program to open it.\n\nDo you want to continue?").arg( url.toString() ),
 				 i18n("&Yes"), i18n("&No"),
 				 QString::null, 0, 1 ) )
 					return false;
@@ -336,59 +414,22 @@ bool MainWindow::openPage( const QString & srcurl, unsigned int flags )
 
 		return false; // do not change the current page.
 	}
-		
-	// Filter the URLs which do not need to be opened at all by Qt version
-	if ( LCHMUrlFactory::isJavascriptURL(url)  )
-	{
-		QMessageBox::information( this, 
-			i18n( "%1 - JavsScript link clicked") . arg(QCoreApplication::applicationName()),
-			i18n( "You have clicked a JavaScript link.\nTo prevent security-related issues JavaScript URLs are disabled in CHM files.") );
-		
-		return false;
-	}
 
-	if ( LCHMUrlFactory::isNewChmURL (url, getOpenedFileName(), otherfile, otherlink) )
-	{
-		if ( otherfile != m_chmFilename )
-		{
-			if ( QMessageBox::question( this,
-				i18n( "%1 - link to a new CHM file clicked"). arg(QCoreApplication::applicationName()),
-				i18n( "You have clicked a link, which leads to a new CHM file %1.\nThe current file will be closed.\n\nDo you want to continue?").arg( otherfile ),
-				i18n( "&Yes" ), i18n( "&No" ),
-				QString::null, 0, 1 ) )
-					return false;
-	
-			// Because chm file always contain relative link, and current filename is not changed,
-			// we need to form a new path
-			QStringList event_args;
-			event_args.push_back( otherfile );
-			event_args.push_back( otherlink ); // url
-			
-			qApp->postEvent( this, new UserEvent( "loadAndOpen", event_args ) );
-			return false;
-		}
-		else
-			url = otherlink;
-	}
-	
 	ViewWindow * vwnd = currentBrowser();
 
 	if ( flags & OPF_NEW_TAB )
 		vwnd = m_viewWindowMgr->addNewTab( !(flags & OPF_BACKGROUND) );
 	
-	// Store current page and position to add it to history if we change it
-	int hist_scrollpos = currentBrowser()->getScrollbarPosition();
-	QString hist_url = currentBrowser()->getOpenedPage();
-	
 	if ( vwnd->openUrl (url) )
 	{
 		// Open all the tree items to show current item (if needed)
 		if ( (flags & OPF_CONTENT_TREE) != 0 )
 			m_navPanel->findUrlInContents( url );
-		
-		if ( flags & OPF_ADD2HISTORY )
-			currentBrowser()->addNavigationHistory( hist_url, hist_scrollpos );
 	}
+
+	// Focus on the view window so keyboard scroll works; do not do it for the background tabs
+	if ( (flags & OPF_BACKGROUND) == 0 )
+		vwnd->setFocus( Qt::OtherFocusReason );
 	
 	return true;
 }
@@ -396,7 +437,7 @@ bool MainWindow::openPage( const QString & srcurl, unsigned int flags )
 
 void MainWindow::firstShow()
 {
-	if ( !parseCmdLineArgs( ) )
+    if ( !parseCmdLineArgs( m_arguments ) )
 	{
 		if ( m_recentFiles && pConfig->m_startupMode == Config::STARTUP_LOAD_LAST_FILE && !m_recentFiles->latestFile().isEmpty() )
 		{
@@ -410,9 +451,9 @@ void MainWindow::firstShow()
 }
 
 
-void MainWindow::setTextEncoding( const LCHMTextEncoding * encoding )
+void MainWindow::setTextEncoding( const QString& encoding )
 {
-	m_chmFile->setCurrentEncoding( encoding );
+	m_ebookFile->setCurrentEncoding( qPrintable( encoding ) );
 	
 	// Find the appropriate encoding item in "Set encodings" menu
 	const QList<QAction *> encodings = m_encodingActions->actions();
@@ -421,9 +462,7 @@ void MainWindow::setTextEncoding( const LCHMTextEncoding * encoding )
 	      it != encodings.end();
 	      ++it )
 	{
-		const LCHMTextEncoding * enc = (const LCHMTextEncoding *) (*it)->data().value< void* > ();
-		
-		if ( !strcmp( enc->qtcodec, encoding->qtcodec ) )
+		if ( (*it)->data().toString() == encoding  )
 		{
 			if ( !(*it)->isChecked() )
 				(*it)->setChecked( true );
@@ -434,7 +473,7 @@ void MainWindow::setTextEncoding( const LCHMTextEncoding * encoding )
 	
 	// Because updateView() will call view->invalidate(), which clears the view->getOpenedPage(),
 	// we have to make a copy of it.
-	QString url = currentBrowser()->getOpenedPage();
+	QUrl url = currentBrowser()->getOpenedPage();
 	
 	// Regenerate the content and index trees	
 	refreshCurrentBrowser();
@@ -447,7 +486,9 @@ void MainWindow::closeFile( )
 	// Prepare the settings
 	if ( pConfig->m_HistoryStoreExtra )
 	{
-		m_currentSettings->m_activeEncoding = m_chmFile->currentEncoding()->qtcodec;
+		if ( m_ebookFile->hasFeature( EBook::FEATURE_ENCODING ) )
+			m_currentSettings->m_activeEncoding = m_ebookFile->currentEncoding();
+
 		m_currentSettings->m_activetabwindow = m_viewWindowMgr->currentPageIndex( );
 		
 		m_currentSettings->m_window_size_x = width();
@@ -468,11 +509,11 @@ void MainWindow::closeFile( )
 void MainWindow::closeEvent ( QCloseEvent * e )
 {
 	// Save the settings if we have something opened
-	if ( m_chmFile )
+	if ( m_ebookFile )
 	{
 		closeFile( );
-		delete m_chmFile;
-		m_chmFile = 0;
+		delete m_ebookFile;
+		m_ebookFile = 0;
 	}
 
 	// Save toolbars
@@ -481,66 +522,92 @@ void MainWindow::closeEvent ( QCloseEvent * e )
 	QMainWindow::closeEvent ( e );
 }
 
-bool MainWindow::parseCmdLineArgs( )
+void MainWindow::printHelpAndExit()
 {
-	QString filename = QString::null, search_query = QString::null;
-	QString search_index = QString::null, open_url = QString::null, search_toc = QString::null;
-	bool do_autotest = false;
+    fprintf (stderr, "Usage: %s [options] [helpfile]\n"
+            "    The following options supported:\n"
+            "  -showPage <url>   opens the url in the help file\n"
+            "  -index <text>     searches for text in the Index tab\n"
+            "  -search <query>   searches for query in the Search tab, and activate the first entry if found\n"
+            "  -token <token>    specifies the application token; see the integration reference\n"
+            "  -background       start minimized\n"
+            "  -novcheck         disable check for new version even if enabled in configuration\n"
+             , qPrintable( m_arguments[0] ) );
 
-#if defined (USE_KDE)
-	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
+    exit (1);
+}
 
-	if ( args->isSet("autotestmode") || args->isSet("shortautotestmode") )
-		do_autotest = true;
+bool MainWindow::parseCmdLineArgs(const QStringList& args , bool from_another_app )
+{
+    QString filename, search_query, search_index, open_url, search_toc;
+    bool do_autotest = false, disable_vcheck = false, force_background = false;
 
-	search_query = args->getOption ("search");
-	search_index = args->getOption ("sindex");
-	search_toc = args->getOption ("stoc");
-	open_url = args->getOption ("url");
-	
-	if ( args->count() > 0 )
-		filename = args->arg(0);
-#else
 	// argv[0] in Qt is still a program name
-	for ( int i = 1; i < qApp->argc(); i++  )
+    for ( int i = 1; i < args.size(); i++  )
 	{
-		if ( !strcmp (qApp->argv()[i], "-h") || !strcmp (qApp->argv()[i], "--help") )
-		{
-			fprintf (stderr, "Usage: %s [options] [chmfile]\n"
-					"    The following options supported:\n"
-					"  --search <query> specifies the search query to search, and activate the first entry if found\n"
-					"  --sindex <word>  specifies the word to find in index, and activate if found\n"
-					"  --stoc <word(s)> specifies the word(s) to find in TOC, and activate if found. Wildcards allowed\n"
-					"  --url <word>     specifies the URL to activate if found.",
-	 				qApp->argv()[0] );
-			
-			exit (1);
-		}
-		else if ( !strcmp (qApp->argv()[i], "--autotestmode") || !strcmp (qApp->argv()[i], "--shortautotestmode") )
+        if ( args[i] == "-h" || args[i] == "--help" )
+            printHelpAndExit();
+        else if ( args[i] == "--autotestmode" || args[i] == "--shortautotestmode" )
 			do_autotest = true;
-		else if ( !strcmp (qApp->argv()[i], "--search") )
-			search_query = qApp->argv()[++i];
-		else if ( !strcmp (qApp->argv()[i], "--sindex") )
-			search_index = qApp->argv()[++i];
-		else if ( !strcmp (qApp->argv()[i], "--stoc") )
-			search_toc = qApp->argv()[++i];
-		else if ( !strcmp (qApp->argv()[i], "--url") )
-			open_url = qApp->argv()[++i];
+        else if ( args[i] == "--search" || args[i] == "-search" )
+            search_query = args[++i];
+        else if ( args[i] == "--sindex" || args[i] == "-index" )
+            search_index = args[++i];
+        else if ( args[i] == "--stoc" )
+            search_toc = args[++i];
+        else if ( args[i] == "-token" )
+            i++; // ignore
+        else if ( args[i] == "-background" )
+            force_background = true;
+        else if ( args[i] == "-novcheck" )
+            disable_vcheck = true;
+        else if ( args[i] == "--url" || args[i] == "-showPage" )
+            open_url = args[++i];
 		else
-			filename = QString::fromLocal8Bit( qApp->argv()[i] );
-	}
-#endif
+        {
+            if ( filename.isEmpty() )
+                filename = args[i];
+            else
+            {
+                // Don't quit just because wrong CL was passed
+                if ( from_another_app )
+                    return false;
 
+                fprintf (stderr, "Invalid command-line option %s (ebook filename is already specified as %s)\n",
+                         qPrintable( filename ), qPrintable( args[i] ) );
+
+                printHelpAndExit();
+            }
+        }
+	}
+
+    // Check for a new version if needed
+    if ( pConfig->m_advCheckNewVersion && !disable_vcheck )
+    {
+        QSettings settings;
+
+        if ( settings.contains( "advanced/lastupdate" ) )
+        {
+            QDateTime lastupdate = settings.value( "advanced/lastupdate" ).toDateTime();
+
+            if ( lastupdate.secsTo( QDateTime::currentDateTime() ) >= 86400 * 7 ) // seven days
+                checkNewVersionAvailable();
+        }
+    }
+
+    // Opening the file?
 	if ( !filename.isEmpty() )
 	{
-		if ( !loadFile( filename ) )
-			return true; // skip the latest checks, but do not exit from the program
+        // If we have already opened the same file, no need to reopen it again
+        if ( !m_ebookFile || QDir(m_ebookFilename) != QDir(filename) )
+            if ( !loadFile( filename ) )
+                return true; // skip the latest checks, but do not exit from the program
 
 		if ( !open_url.isEmpty() )
 		{
 			QStringList event_args;
-			event_args.push_back( open_url );
-			qApp->postEvent( this, new UserEvent( "openPage", event_args ) );
+            event_args.push_back( m_ebookFile->pathToUrl(open_url).toString() );
+            qApp->postEvent( this, new UserEvent( "openPage", event_args ) );
 		}
 		else if ( !search_index.isEmpty() )
 		{
@@ -570,6 +637,36 @@ bool MainWindow::parseCmdLineArgs( )
 			showMinimized ();
 			runAutoTest();
 		}
+
+        if ( force_background )
+            showMinimized();
+        else if ( from_another_app )
+        {
+#ifdef Q_WS_X11
+            // On Linux - at least on KDE - activating the foreground window
+            // via activateWindow(); raise(); only works twice. Then it does not
+            // work anymore, most likely because of some internal counter in Qt.
+            // The code below, however, works fine.
+            Display * display = x11Info().display();
+            WId win = winId();
+
+            XEvent event = { 0 };
+            event.xclient.type = ClientMessage;
+            event.xclient.serial = 0;
+            event.xclient.send_event = True;
+            event.xclient.message_type = XInternAtom( display, "_NET_ACTIVE_WINDOW", False);
+            event.xclient.window = win;
+            event.xclient.format = 32;
+
+            XSendEvent( display, DefaultRootWindow(display), False, SubstructureRedirectMask | SubstructureNotifyMask, &event );
+            XMapRaised( display, win );
+#else
+            activateWindow();
+            raise();
+            show();
+#endif
+        }
+
 		return true;
 	}
 	
@@ -713,12 +810,12 @@ void MainWindow::actionNavigateHome()
 void MainWindow::actionOpenFile()
 {
 #if defined (USE_KDE)
-	QString fn = KFileDialog::getOpenFileName( pConfig->m_lastOpenedDir, i18n("*.chm|Compressed Help Manual (*.chm)"), this);
+	QString fn = KFileDialog::getOpenFileName( pConfig->m_lastOpenedDir, i18n("*.chm|Compressed Help Manual;*.epub|EPUB electronic book"), this);
 #else
 	QString fn = QFileDialog::getOpenFileName( this, 
 	                                           i18n( "Open a chm file"), 
 											   pConfig->m_lastOpenedDir,
-	                                           i18n("Compressed Help Manual (*.chm)"),
+											   i18n("Electronic books (*.chm *.epub)"),
 	                                           0,
 	                                           QFileDialog::DontResolveSymlinks );
 #endif
@@ -757,7 +854,7 @@ void MainWindow::actionChangeSettings()
 
 void MainWindow::actionExtractCHM()
 {
-	QStringList files;
+	QList< QUrl > files;
 	
 #if defined (USE_KDE)
 	QString outdir = KFileDialog::getExistingDirectory (
@@ -778,7 +875,7 @@ void MainWindow::actionExtractCHM()
 	outdir += "/";
 	
 	// Enumerate all the files in archive
-	if ( !m_chmFile || !m_chmFile->enumerateFiles( &files ) )
+	if ( !m_ebookFile || !m_ebookFile->enumerateFiles( files ) )
 		return;
 
 	KQProgressModalDialog progress( i18n("Extracting CHM content"), 
@@ -802,10 +899,10 @@ void MainWindow::actionExtractCHM()
 		// Extract the file
 		QByteArray buf;
 		
-		if ( m_chmFile->getFileContentAsBinary( &buf, files[i] ) )
+		if ( m_ebookFile->getFileContentAsBinary( buf, files[i] ) )
 		{
 			// Split filename to get the list of subdirectories
-			QStringList dirs = files[i].split( '/' );
+			QStringList dirs = files[i].path().split( '/' );
 
 			// Walk through the list of subdirectories, and create them if needed
 			// dirlevel is used to detect extra .. and prevent overwriting files
@@ -849,7 +946,7 @@ void MainWindow::actionExtractCHM()
 			wf.close();
 		}
 		else
-			qWarning( "Could not get file %s\n", qPrintable( files[i] ) );
+			qWarning( "Could not get file %s\n", qPrintable( files[i].toString() ) );
 	}
 	
 	progress.setValue( files.size() );
@@ -869,14 +966,14 @@ void MainWindow::actionViewHTMLsource()
 {
 	QString text;
 
-	if ( !m_chmFile->getFileContentAsString( &text, currentBrowser()->getOpenedPage() ) || text.isEmpty() )
+	if ( !m_ebookFile->getFileContentAsString( text, currentBrowser()->getOpenedPage() ) || text.isEmpty() )
 		return;
 
 	if ( pConfig->m_advUseInternalEditor )
 	{
 		QTextEdit * editor = new QTextEdit ( 0 );
 		editor->setPlainText( text );
-		editor->setWindowTitle( QString("HTML source of %1") .arg( currentBrowser()->getOpenedPage() ));
+		editor->setWindowTitle( i18n("HTML source") );
 		editor->resize( 800, 600 );
 		editor->show();
 	}
@@ -959,9 +1056,9 @@ void MainWindow::actionLocateInContentsTab()
 void MainWindow::actionAboutApp()
 {
 	QString abouttext = i18n( "<html><b>kchmviewer version %1.%2</b><br><br>"
-							  "Copyright (C) George Yunaev, 2004-2010<br>"
+							  "Copyright (C) George Yunaev, 2004-2014<br>"
 							  "<a href=\"mailto:gyunaev@ulduzsoft.com\">gyunaev@ulduzsoft.com</a><br>"
-							  "<a href=\"http://www.kchmviewer.net\">http://www.kchmviewer.net</a><br><br>"
+							  "<a href=\"http://www.ulduzsoft.com/kchmviewer\">http://www.ulduzsoft.com/kchmviewer</a><br><br>"
 							  "Licensed under GNU GPL license version 3.</html>" )
 								.arg(APP_VERSION_MAJOR) .arg(APP_VERSION_MINOR);
 
@@ -1093,13 +1190,26 @@ void MainWindow::setupActions()
 	                      SLOT( onActivateFind() ),
 	                      Qt::ApplicationShortcut );
 	
-	// Find next global shortcut
+	// Find next global shortcuts
 	(void) new QShortcut( QKeySequence( i18n("F3") ),
 	                      m_viewWindowMgr,
 	                      SLOT( onFindNext() ),
 	                      SLOT( onFindNext() ),
 	                      Qt::ApplicationShortcut );
 
+	(void) new QShortcut( QKeySequence( QKeySequence::FindNext ),
+						  m_viewWindowMgr,
+						  SLOT( onFindNext() ),
+						  SLOT( onFindNext() ),
+						  Qt::ApplicationShortcut );
+
+	// Find prev global shortcut
+	(void) new QShortcut( QKeySequence( QKeySequence::FindPrevious ),
+						  m_viewWindowMgr,
+						  SLOT( onFindPrevious() ),
+						  SLOT( onFindPrevious() ),
+						  Qt::ApplicationShortcut );
+
 	// Open next page in TOC global shortcut
 	(void) new QShortcut( QKeySequence( i18n("Ctrl+Right") ),
 						  m_navPanel,
@@ -1184,18 +1294,17 @@ void MainWindow::setupLangEncodingMenu()
 	// Create the action group
 	m_encodingActions = new QActionGroup( this );
 	
-	// Add the codepage entries
-	const LCHMTextEncoding * enctable = LCHMFileImpl::getTextEncodingTable();
-	
-	for ( int idx = 0; (enctable + idx)->family; idx++ )
+	// Get the supported languages and encodings
+	QStringList languages, qencodings;
+	TextEncodings::getSupported( languages, qencodings );
+
+	for ( int idx = 0; idx < qencodings.size(); idx++ )
 	{
-		const LCHMTextEncoding * enc = enctable + idx;
-		
 		QAction * action = new QAction( this );
 		
-		QString text = i18n("%1 ( %2 )") .arg( enc->family) .arg( enc->qtcodec );
+		QString text = i18n("%1 ( %2 )") .arg( languages[idx] ) .arg( qencodings[idx] );
 		action->setText( text );
-		action->setData( qVariantFromValue( (void*) enc ) );
+		action->setData( qVariantFromValue( qencodings[idx] ) );
 		action->setCheckable( true );
 		
 		// Add to the action group, so only one is checkable
@@ -1218,8 +1327,8 @@ void MainWindow::setupLangEncodingMenu()
 
 void MainWindow::actionEncodingChanged( QAction * action )
 {
-	const LCHMTextEncoding * enc = (const LCHMTextEncoding *) action->data().value< void* > ();
-	setTextEncoding( enc );
+	QString encoding = action->data().toString();
+	setTextEncoding( encoding );
 }
 
 
@@ -1248,17 +1357,30 @@ void MainWindow::setupPopupMenu( QMenu * menu )
 
 bool MainWindow::hasTableOfContents() const
 {
-	return m_chmFile && m_chmFile->hasTableOfContents();
+	return m_ebookFile && m_ebookFile->hasFeature( EBook::FEATURE_TOC );
 }
 
 bool MainWindow::hasIndex() const
 {
-	return m_chmFile && m_chmFile->hasIndexTable();
+	return m_ebookFile && m_ebookFile->hasFeature( EBook::FEATURE_INDEX );
+}
+
+const QPixmap *MainWindow::getEBookIconPixmap(EBookTocEntry::Icon imagenum)
+{
+	if ( m_builtinIcons[imagenum].isNull() )
+	{
+		QString resicon = QString( ":/chm_icons/icon_%1.png") .arg( imagenum );
+
+		if ( !m_builtinIcons[imagenum].load( resicon ) )
+			qFatal("Could not initialize the internal icon %d as %s", imagenum, qPrintable( resicon ) );
+	}
+
+	return &(m_builtinIcons[imagenum]);
 }
 
 void MainWindow::updateActions()
 {
-	bool enabled = m_chmFile != 0;
+	bool enabled = m_ebookFile != 0;
 
 	file_Print_action->setEnabled( enabled );
 	edit_Copy_action->setEnabled( enabled );
@@ -1292,7 +1414,7 @@ void MainWindow::newVerAvailable( NewVersionMetaMap metadata )
 	// What is the latest version?
 	QString current = QString("%1.%2") .arg(APP_VERSION_MAJOR) .arg(APP_VERSION_MINOR);
 
-	if ( metadata["Version"] != current )
+    if ( metadata["Version"].toFloat() > current.toFloat() )
 	{
 		if ( QMessageBox::question( 0,
 				tr("New version available"),
