@@ -1,43 +1,36 @@
-/**************************************************************************
- *  Kchmviewer - a portable CHM file viewer with the best support for     *
- *  the international languages                                           *
- *                                                                        *
- *  Copyright (C) 2004-2012 George Yunaev, kchmviewer@ulduzsoft.com       *
- *                                                                        *
- *  Please read http://www.kchmviewer.net/reportbugs.html if you want     *
- *  to report a bug. It lists things I need to fix it!                    *
- *                                                                        *
- *  This program is free software: you can redistribute it and/or modify  *
- *  it under the terms of the GNU General Public License as published by  *
- *  the Free Software Foundation, either version 3 of the License, or     *
- *  (at your option) any later version.                                   *
- *																	      *
- *  This program is distributed in the hope that it will be useful,       *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *  GNU General Public License for more details.                          *
- *                                                                        *
- *  You should have received a copy of the GNU General Public License     *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- **************************************************************************/
+/*
+ *  Kchmviewer - a CHM and EPUB file viewer with broad language support
+ *  Copyright (C) 2004-2014 George Yunaev, gyunaev@ulduzsoft.com
+ *
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include <QHeaderView>
 
-#include "libchmfile.h"
-
 #include "mainwindow.h"
 #include "config.h"
-#include "treeviewitem.h"
 #include "tab_search.h"
+#include "ebook_search.h"
 
 
 class SearchTreeViewItem : public QTreeWidgetItem
 {
 	public:
-		SearchTreeViewItem( QTreeWidget * tree, const QString& name, const QString& url )
+		SearchTreeViewItem( QTreeWidget * tree, const QString& name, const QUrl& url )
 			:	QTreeWidgetItem( tree ), m_name( name ), m_url( url ) {}
 	
-		QString		getUrl() const { return m_url; }
+		QUrl	getUrl() const { return m_url; }
 	
 	protected:
 		// Overriden members
@@ -55,7 +48,7 @@ class SearchTreeViewItem : public QTreeWidgetItem
 				if ( column == 0 )
 					return m_name;
 				else
-					return m_url;
+					return m_url.path();
 			}
 			
 			return QVariant();
@@ -63,7 +56,7 @@ class SearchTreeViewItem : public QTreeWidgetItem
 	
 	private:
 		QString		m_name;		
-		QString		m_url;
+		QUrl		m_url;
 };
 
 
@@ -94,9 +87,9 @@ TabSearch::TabSearch( QWidget * parent )
 	
 	// Clicking on tree element
 	connect( tree, 
-	         SIGNAL( itemDoubleClicked( QTreeWidgetItem *, int ) ), 
+			 SIGNAL( itemActivated( QTreeWidgetItem *, int ) ),
 			 this, 
-	         SLOT( onDoubleClicked( QTreeWidgetItem *, int ) ) );
+			 SLOT( onItemActivated( QTreeWidgetItem *, int ) ) );
 
 	// Activate custom context menu, and connect it
 	tree->setContextMenuPolicy( Qt::CustomContextMenu );
@@ -105,13 +98,13 @@ TabSearch::TabSearch( QWidget * parent )
 			 this, 
 			 SLOT( onContextMenuRequested( const QPoint & ) ) );
 
-	searchBox->setFocus();
+	focus();
 	
 	m_contextMenu = 0;
 	m_genIndexProgress = 0;
 	m_searchEngineInitDone = false;
 	
-	m_searchEngine = new LCHMSearchEngine();
+	m_searchEngine = new EBookSearch();
 	connect( m_searchEngine, SIGNAL( progressStep( int, const QString& ) ), this, SLOT( onProgressStep( int, const QString& ) ) );
 }
 
@@ -131,7 +124,7 @@ void TabSearch::invalidate( )
 
 void TabSearch::onReturnPressed( )
 {
-	QStringList results;
+	QList<QUrl> results;
 	QString text = searchBox->lineEdit()->text();
 	
 	if ( text.isEmpty() )
@@ -145,12 +138,16 @@ void TabSearch::onReturnPressed( )
 		{
 			for ( int i = 0; i < results.size(); i++ )
 			{
-				new SearchTreeViewItem ( tree,
-										 ::mainWindow->chmFile()->getTopicByUrl( results[i] ),
-										 results[i] );
+				SearchTreeViewItem * item = new SearchTreeViewItem ( tree,
+																	 ::mainWindow->chmFile()->getTopicByUrl( results[i] ),
+																	 results[i] );
+
+				if ( i == 0 )
+					tree->setCurrentItem( item );
 			}
 
 			::mainWindow->showInStatusBar( i18n( "Search returned %1 result(s)" ) . arg(results.size()) );
+			tree->setFocus();
 		}
 		else
 			::mainWindow->showInStatusBar( i18n( "Search returned no results") );
@@ -160,13 +157,13 @@ void TabSearch::onReturnPressed( )
 }
 
 
-void TabSearch::onDoubleClicked( QTreeWidgetItem * item, int )
+void TabSearch::onItemActivated( QTreeWidgetItem * item, int )
 {
 	if ( !item )
 		return;
 	
 	SearchTreeViewItem * treeitem = (SearchTreeViewItem *) item;
-	::mainWindow->openPage( treeitem->getUrl(), MainWindow::OPF_ADD2HISTORY );
+	::mainWindow->currentBrowser()->openUrl( treeitem->getUrl() );
 }
 
 
@@ -266,7 +263,7 @@ void TabSearch::execSearchQueryInGui( const QString & query )
 }
 
 
-bool TabSearch::searchQuery( const QString & query, QStringList * results )
+bool TabSearch::searchQuery( const QString & query, QList< QUrl > * results )
 {
 	if ( !m_searchEngineInitDone )
 	{
@@ -290,6 +287,11 @@ bool TabSearch::searchQuery( const QString & query, QStringList * results )
 	return result;
 }
 
+void TabSearch::focus()
+{
+	if ( !searchBox->hasFocus() && !tree->hasFocus() )
+		searchBox->setFocus();
+}
 
 void TabSearch::onContextMenuRequested( const QPoint & point )
 {
