@@ -1,26 +1,20 @@
-/**************************************************************************
- *  Kchmviewer - a portable CHM file viewer with the best support for     *
- *  the international languages                                           *
- *                                                                        *
- *  Copyright (C) 2004-2012 George Yunaev, kchmviewer@ulduzsoft.com       *
- *                                                                        *
- *  Please read http://www.kchmviewer.net/reportbugs.html if you want     *
- *  to report a bug. It lists things I need to fix it!                    *
- *                                                                        *
- *  This program is free software: you can redistribute it and/or modify  *
- *  it under the terms of the GNU General Public License as published by  *
- *  the Free Software Foundation, either version 3 of the License, or     *
- *  (at your option) any later version.                                   *
- *																	      *
- *  This program is distributed in the hope that it will be useful,       *
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
- *  GNU General Public License for more details.                          *
- *                                                                        *
- *  You should have received a copy of the GNU General Public License     *
- *  along with this program.  If not, see <http://www.gnu.org/licenses/>. *
- **************************************************************************/
-
+/*
+ *  Kchmviewer - a CHM and EPUB file viewer with broad language support
+ *  Copyright (C) 2004-2014 George Yunaev, gyunaev@ulduzsoft.com
+ *
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "config.h"
 #include "mainwindow.h"
@@ -85,9 +79,13 @@ ViewWindowMgr::ViewWindowMgr( QWidget *parent )
 	         SIGNAL( textEdited ( const QString & ) ),
 	         this, 
 	         SLOT( editTextEdited( const QString & ) ) );
-	
+
+    connect( editFind, SIGNAL(returnPressed()), this, SLOT(onFindNext()) );
+
 	// Search toolbar buttons
-	connect( toolClose, SIGNAL(clicked()), frameFind, SLOT( hide()) );
+	toolClose->setShortcut( Qt::Key_Escape );
+	connect( toolClose, SIGNAL(clicked()), this, SLOT( closeSearch()) );
+
 	connect( toolPrevious, SIGNAL(clicked()), this, SLOT( onFindPrevious()) );
 	connect( toolNext, SIGNAL(clicked()), this, SLOT( onFindNext()) );
 }
@@ -172,7 +170,7 @@ void ViewWindowMgr::setTabName( ViewWindow * window )
 	
 	if ( tab )
 	{
-		QString title = window->getTitle();
+		QString title = window->getTitle().trimmed();
 		
 		// Trim too long string
 		if ( title.length() > 25 )
@@ -262,7 +260,7 @@ void ViewWindowMgr::saveSettings( Settings::viewindow_saved_settings_t & setting
 		if ( !tab )
 			abort();
 		
-		settings.push_back( Settings::SavedViewWindow( tab->window->getOpenedPage(),
+		settings.push_back( Settings::SavedViewWindow( tab->window->getOpenedPage().toString(),
 													   tab->window->getScrollbarPosition(),
 													   tab->window->getZoomFactor()) );
 	}
@@ -285,7 +283,7 @@ void ViewWindowMgr::onTabChanged( int newtabIndex )
 	
 	if ( tab )
 	{
-		tab->window->updateNavigationToolbar();
+		tab->window->updateHistoryIcons();
 		mainWindow->browserChanged( tab->window );
 		tab->widget->setFocus();
 	}
@@ -294,8 +292,7 @@ void ViewWindowMgr::onTabChanged( int newtabIndex )
 
 void ViewWindowMgr::openNewTab()
 {
-	::mainWindow->openPage( current()->getOpenedPage(), 
-							MainWindow::OPF_NEW_TAB | MainWindow::OPF_CONTENT_TREE | MainWindow::OPF_ADD2HISTORY );
+	::mainWindow->openPage( current()->getOpenedPage(), MainWindow::OPF_NEW_TAB | MainWindow::OPF_CONTENT_TREE );
 }
 
 void ViewWindowMgr::activateWindow()
@@ -313,6 +310,12 @@ void ViewWindowMgr::activateWindow()
 	}
 }
 
+void ViewWindowMgr::closeSearch()
+{
+	frameFind->hide();
+	m_tabWidget->currentWidget()->setFocus();
+}
+
 ViewWindowMgr::TabData * ViewWindowMgr::findTab(QWidget * widget)
 {
 	for ( WindowsIterator it = m_Windows.begin(); it != m_Windows.end(); ++it )
@@ -342,29 +345,61 @@ void ViewWindowMgr::onActivateFind()
 
 void ViewWindowMgr::find( bool backward )
 {
-	int flags = 0;
+    QWebPage::FindFlags webkitflags = 0;
 	
 	if ( checkCase->isChecked() )
-		flags |= ViewWindow::SEARCH_CASESENSITIVE;
+        webkitflags |= QWebPage::FindCaseSensitively;
 	
 	if ( backward )
-		flags |= ViewWindow::SEARCH_BACKWARD;
+        webkitflags |= QWebPage::FindBackward;
+
+    if ( pConfig->m_browserHighlightSearchResults )
+    {
+        // From the doc:
+        // If the HighlightAllOccurrences flag is passed, the
+        // function will highlight all occurrences that exist
+        // in the page. All subsequent calls will extend the
+        // highlight, rather than replace it, with occurrences
+        // of the new string.
+
+        // If the search text is different, we run the empty string search
+        // to discard old highlighting
+        if ( m_lastSearchedWord != editFind->text() )
+            current()->findText( "", webkitflags | QWebPage::HighlightAllOccurrences );
+
+        m_lastSearchedWord = editFind->text();
+
+        // Now we call search with highlighting enabled, while the main search below will have
+        // it disabled. This leads in both having the highlighting results AND working forward/
+        // backward buttons.
+        current()->findText( editFind->text(), webkitflags | QWebPage::HighlightAllOccurrences );
+    }
+
+    // Pre-hide the wrapper
+    labelWrapped->hide();
+
+    bool found = current()->findText( editFind->text(), webkitflags );
+
+    // If we didn't find anything, enable the wrap and try again
+    if ( !found )
+    {
+        found = current()->findText( editFind->text(), webkitflags | QWebPage::FindWrapsAroundDocument );
+
+        if ( found )
+            labelWrapped->show();
+    }
 
-	// Remember the existing selection position to check the wraps
-	bool res = current()->findTextInPage( editFind->text(), flags );
-		
 	if ( !frameFind->isVisible() )
 		frameFind->show();
 
 	QPalette p = editFind->palette();
 
-	if ( !res )
+    if ( !found )
 		p.setColor( QPalette::Active, QPalette::Base, QColor(255, 102, 102) );
 	else
 		p.setColor( QPalette::Active, QPalette::Base, Qt::white );
 
 	editFind->setPalette( p );
-	//labelWrapped->setVisible( status == SearchResultFoundWrapped );
 }
 
 
